package sun.awt.image;

import java.security.AccessController;
import java.security.AccessControlContext;
import java.lang.ref.WeakReference;
import java.awt.Image;
import java.awt.image.ImageObserver;

public abstract class ImageWatched
{
    public static ImageWatched.Link endlink;
    public ImageWatched.Link watcherList;
    
    public ImageWatched() {
        this.watcherList = ImageWatched.endlink;
    }
    
    public synchronized void addWatcher(ImageObserver imageObserver) {
        if (imageObserver != null && !this.isWatcher(imageObserver)) {
            this.watcherList = new ImageWatched.WeakLink(imageObserver, this.watcherList);
        }
        this.watcherList = this.watcherList.removeWatcher(null);
    }
    
    public synchronized boolean isWatcher(ImageObserver imageObserver) {
        return this.watcherList.isWatcher(imageObserver);
    }
    
    public void removeWatcher(ImageObserver imageObserver) {
        synchronized (this) {
            this.watcherList = this.watcherList.removeWatcher(imageObserver);
        }
        if (this.watcherList == ImageWatched.endlink) {
            this.notifyWatcherListEmpty();
        }
    }
    
    public boolean isWatcherListEmpty() {
        synchronized (this) {
            this.watcherList = this.watcherList.removeWatcher(null);
        }
        return this.watcherList == ImageWatched.endlink;
    }
    
    public void newInfo(Image image, int n, int n2, int n3, int n4, int n5) {
        if (this.watcherList.newInfo(image, n, n2, n3, n4, n5)) {
            this.removeWatcher(null);
        }
    }
    
    protected abstract void notifyWatcherListEmpty();
    
    static {
        ImageWatched.endlink = new ImageWatched.Link();
    }
    
    public static class Link
    {
        public boolean isWatcher(ImageObserver imageObserver) {
            return false;
        }
        
        public ImageWatched.Link removeWatcher(ImageObserver imageObserver) {
            return this;
        }
        
        public boolean newInfo(Image image, int n, int n2, int n3, int n4, int n5) {
            return false;
        }
    }
    
    static class AccWeakReference<T> extends WeakReference<T>
    {
        private final AccessControlContext acc;
        
        AccWeakReference(T referent) {
            super(referent);
            this.acc = AccessController.getContext();
        }
    }
    
    public static class WeakLink extends ImageWatched.Link
    {
        private final ImageWatched.AccWeakReference<ImageObserver> myref;
        private ImageWatched.Link next;
        
        public WeakLink(ImageObserver imageObserver, ImageWatched.Link next) {
            this.myref = new ImageWatched.AccWeakReference(imageObserver);
            this.next = next;
        }
        
        @Override
        public boolean isWatcher(ImageObserver imageObserver) {
            return this.myref.get() == imageObserver || this.next.isWatcher(imageObserver);
        }
        
        @Override
        public ImageWatched.Link removeWatcher(ImageObserver imageObserver) {
            ImageObserver imageObserver2 = (ImageObserver)this.myref.get();
            if (imageObserver2 == null) {
                return this.next.removeWatcher(imageObserver);
            }
            if (imageObserver2 == imageObserver) {
                return this.next;
            }
            this.next = this.next.removeWatcher(imageObserver);
            return this;
        }
        
        private static boolean update(ImageObserver imageObserver, AccessControlContext context, Image image, int n, int n2, int n3, int n4, int n5) {
            return (context != null || System.getSecurityManager() != null) && (boolean)AccessController.doPrivileged(()/*  */ -> Boolean.valueOf(p0.imageUpdate(p2, p3, p4, p5, p6, p7)), context);
        }
        
        @Override
        public boolean newInfo(Image image, int n, int n2, int n3, int n4, int n5) {
            int info = this.next.newInfo(image, n, n2, n3, n4, n5);
            ImageObserver imageObserver = (ImageObserver)this.myref.get();
            if (imageObserver == null) {
                info = true;
            }
            else if (!update(imageObserver, this.myref.acc, image, n, n2, n3, n4, n5)) {
                this.myref.clear();
                info = true;
            }
            return info;
        }
    }
}
